@@ -1,11 +1,11 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer
- version: 2.2
+ version: 2.4
  Written in Qt
 
                          dialogsettings.cpp  -  description
                              -------------------
-    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    copyright         : (C) 2005-2018 by Dimitris B. Kalamaras
     project site      : http://socnetv.org
 
  ***************************************************************************/
@@ -48,31 +48,20 @@ DialogSettings::DialogSettings(
     //data export
     ui->dataDirEdit->setText(  (m_appSettings)["dataDir"]);
 
+    ui->printLogoChkBox->setChecked(
+                (appSettings["printLogo"] == "true") ? true:false
+                );
+
+
     //debugging
     ui->printDebugChkBox->setChecked(
                 (appSettings["printDebug"] == "true") ? true:false
             );
 
-    ui->progressBarsChkBox->setChecked(
+    ui->progressDialogChkBox->setChecked(
                 (appSettings["showProgressBar"] == "true") ? true:false
                 );
 
-    /**
-     * canvas options
-     */
-    ui->antialiasingChkBox->setChecked(
-                (appSettings["antialiasing"] == "true") ? true:false
-                );
-    ui->printLogoChkBox->setChecked(
-                (appSettings["printLogo"] == "true") ? true:false
-                );
-
-    m_bgColor = QColor (m_appSettings["initBackgroundColor"]);
-    m_pixmap = QPixmap(60,20) ;
-    m_pixmap.fill( m_bgColor );
-    ui->bgColorButton->setIcon(QIcon(m_pixmap));
-
-    ui->bgImageSelectEdit->setText((m_appSettings)["initBackgroundImage"]);
 
     /**
      * window options
@@ -85,6 +74,87 @@ DialogSettings::DialogSettings(
                 ( appSettings["showRightPanel"] == "true") ? true:false
                 );
 
+
+    /**
+     * canvas options
+     */
+
+    m_bgColor = QColor (m_appSettings["initBackgroundColor"]);
+    m_pixmap = QPixmap(60,20) ;
+    m_pixmap.fill( m_bgColor );
+    ui->bgColorButton->setIcon(QIcon(m_pixmap));
+
+    ui->bgImageSelectEdit->setText((m_appSettings)["initBackgroundImage"]);
+
+
+    ui->canvasAntialiasingChkBox->setChecked(
+                (appSettings["antialiasing"] == "true") ? true:false
+                );
+
+    ui->canvasAntialiasingAutoAdjustChkBox->setChecked(
+                (appSettings["canvasAntialiasingAutoAdjustment"] == "true") ? true:false
+                );
+    ui->canvasSmoothPixmapTransformChkBox->setChecked(
+                (appSettings["canvasSmoothPixmapTransform"] == "true") ? true:false
+                );
+
+    ui->canvasSavePainterStateChkBox->setChecked(
+                (appSettings["canvasPainterStateSave"] == "true") ? true:false
+                );
+
+    ui->canvasCacheBackgroundChkBox->setChecked(
+                (appSettings["canvasCacheBackground"] == "true") ? true:false
+                );
+    ui->canvasEdgeHighlightingChkBox->setChecked(
+                (appSettings["canvasEdgeHighlighting"] == "true") ? true:false
+                );
+
+
+    QStringList optionsList;
+    optionsList << "Full" << "Minimal" << "Smart" << "Bounding Rectangle" << "None";
+    ui->canvasUpdateModeSelect->addItems(optionsList);
+
+    if ( appSettings["canvasUpdateMode"] == "Full" ) {
+        ui->canvasUpdateModeSelect->setCurrentText( "Full");
+    }
+    else if (appSettings["canvasUpdateMode"] == "Minimal" ) {
+        ui->canvasUpdateModeSelect->setCurrentText("Minimal" );
+    }
+    else if (appSettings["canvasUpdateMode"] == "Smart" ) {
+        ui->canvasUpdateModeSelect->setCurrentText("Smart" );
+    }
+    else if (appSettings["canvasUpdateMode"] == "Bounding Rectangle" ) {
+        ui->canvasUpdateModeSelect->setCurrentText("Bounding Rectangle" );
+    }
+    else if (appSettings["canvasUpdateMode"] == "None" ) {
+        ui->canvasUpdateModeSelect->setCurrentText("None" );
+    }
+    else { //
+        ui->canvasUpdateModeSelect->setCurrentText("Minimal" );
+    }
+    qDebug() << "canvasUpdateModeSelect" << appSettings["canvasUpdateMode"];
+
+
+    optionsList.clear();
+    optionsList << "BspTreeIndex" << "NoIndex" ;
+    ui->canvasIndexMethodSelect->addItems(optionsList);
+
+    if ( appSettings["canvasIndexMethod"] == "BspTreeIndex" ) {
+        ui->canvasIndexMethodSelect->setCurrentText( "BspTreeIndex");
+    }
+    else if (appSettings["canvasIndexMethod"] == "NoIndex" ) {
+        ui->canvasIndexMethodSelect->setCurrentText("NoIndex" );
+    }
+    else { //
+        ui->canvasIndexMethodSelect->setCurrentText("BspTreeIndex" );
+    }
+    qDebug() << "canvasIndexMethodSelect" << appSettings["canvasIndexMethod"];
+
+
+
+
+
+
     /**
      * node options
      */
@@ -169,12 +239,20 @@ DialogSettings::DialogSettings(
     m_pixmap.fill( m_edgeColor );
     ui->edgeColorBtn->setIcon(QIcon(m_pixmap));
 
+
     m_edgeColorNegative = QColor (m_appSettings["initEdgeColorNegative"]);
     m_pixmap = QPixmap(60,20) ;
     m_pixmap.fill( m_edgeColorNegative );
     ui->edgeColorNegativeBtn ->setIcon(QIcon(m_pixmap));
 
 
+    m_edgeColorZero = QColor (m_appSettings["initEdgeColorZero"]);
+    m_pixmap = QPixmap(60,20) ;
+    m_pixmap.fill( m_edgeColorZero);
+    ui->edgeColorZeroBtn ->setIcon(QIcon(m_pixmap));
+
+
+
     if (m_appSettings["initEdgeShape"] == "line") {
         ui->edgeShapeRadioStraightLine->setChecked(true);
     }
@@ -186,6 +264,8 @@ DialogSettings::DialogSettings(
     }
 
 
+    ui->edgeOffsetFromNodeSpin->setValue( m_appSettings["initEdgeOffsetFromNode"].toInt(0, 10) );
+
     ui->edgeWeightNumbersChkBox-> setChecked(
                 (m_appSettings["initEdgeWeightNumbersVisibility"] == "true") ? true: false
                                                                   );
@@ -209,20 +289,13 @@ DialogSettings::DialogSettings(
     connect (ui->printDebugChkBox, &QCheckBox::stateChanged,
              this, &DialogSettings::setDebugMsgs);
 
-    connect (ui->antialiasingChkBox, &QCheckBox::stateChanged,
-             this, &DialogSettings::setAntialiasing);
 
     connect (ui->printLogoChkBox, &QCheckBox::stateChanged,
              this, &DialogSettings::setPrintLogo);
 
-    connect (ui->bgColorButton, &QToolButton::clicked,
-             this, &DialogSettings::getBgColor);
 
-    connect (ui->bgImageSelectButton, &QToolButton::clicked,
-             this, &DialogSettings::getBgImage);
-
-    connect (ui->progressBarsChkBox, &QCheckBox::stateChanged,
-             this, &DialogSettings::setProgressBars);
+    connect (ui->progressDialogChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setProgressDialog);
 
     connect (ui->showToolBarChkBox, &QCheckBox::stateChanged,
              this, &DialogSettings::setToolBar);
@@ -236,6 +309,46 @@ DialogSettings::DialogSettings(
     connect (ui->rightPanelChkBox, &QCheckBox::stateChanged,
              this, &DialogSettings::setRightPanel);
 
+
+    connect (ui->bgColorButton, &QToolButton::clicked,
+             this, &DialogSettings::getCanvasBgColor);
+
+    connect (ui->bgImageSelectButton, &QToolButton::clicked,
+             this, &DialogSettings::getCanvasBgImage);
+
+
+    connect (ui->canvasAntialiasingChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setCanvasAntialiasing);
+
+    connect (ui->canvasAntialiasingAutoAdjustChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setCanvasAntialiasingAutoAdjust);
+
+    connect (ui->canvasSmoothPixmapTransformChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setCanvasSmoothPixmapTransform);
+
+
+    connect (ui->canvasSavePainterStateChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setCanvasSavePainterState);
+
+    connect (ui->canvasCacheBackgroundChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setCanvasCacheBackground);
+
+
+    connect (ui->canvasEdgeHighlightingChkBox, &QCheckBox::stateChanged,
+             this, &DialogSettings::setCanvasEdgeHighlighting);
+
+
+    connect(ui->canvasUpdateModeSelect, SIGNAL ( currentIndexChanged (const QString &)),
+          this, SLOT(getCanvasUpdateMode(const QString &)) );
+
+
+    connect(ui->canvasIndexMethodSelect, SIGNAL ( currentIndexChanged (const QString &)),
+          this, SLOT(getCanvasIndexMethod(const QString &)) );
+
+
+
+
+
     connect (ui->nodeShapeRadioBox, &QRadioButton::clicked,
              this, &DialogSettings::getNodeShape);
     connect (ui->nodeShapeRadioCircle, &QRadioButton::clicked,
@@ -253,9 +366,6 @@ DialogSettings::DialogSettings(
     connect(ui->nodeSizeSpin, SIGNAL(valueChanged(int)),
             this, SLOT(getNodeSize(int)) );
 
-    connect ( ui->buttonBox, &QDialogButtonBox::accepted,
-              this, &DialogSettings::validateSettings );
-
     connect (ui->nodeColorBtn, &QToolButton::clicked,
              this, &DialogSettings::getNodeColor);
 
@@ -289,11 +399,17 @@ DialogSettings::DialogSettings(
              this, &DialogSettings::getEdgeColor);
     connect (ui->edgeColorNegativeBtn, &QToolButton::clicked,
              this, &DialogSettings::getEdgeColorNegative);
+    connect (ui->edgeColorZeroBtn, &QToolButton::clicked,
+             this, &DialogSettings::getEdgeColorZero);
+
     connect (ui->edgeShapeRadioStraightLine, &QRadioButton::clicked,
              this, &DialogSettings::getEdgeShape);
     connect (ui->edgeShapeRadioBezier, &QRadioButton::clicked,
              this, &DialogSettings::getEdgeShape);
 
+    connect(ui->edgeOffsetFromNodeSpin, SIGNAL(valueChanged(int)),
+            this, SLOT(getEdgeOffsetFromNode(int)) );
+
     connect (ui->edgeWeightNumbersChkBox, &QCheckBox::stateChanged,
                      this, &DialogSettings::getEdgeWeightNumbersVisibility);
 
@@ -301,6 +417,9 @@ DialogSettings::DialogSettings(
     connect (ui->edgeLabelsChkBox, &QCheckBox::stateChanged,
                      this, &DialogSettings::getEdgeLabelsVisibility);
 
+    connect ( ui->buttonBox, &QDialogButtonBox::accepted,
+              this, &DialogSettings::validateSettings );
+
 }
 
 
@@ -331,10 +450,10 @@ void DialogSettings::getDataDir(){
 
 
 /**
- * @brief DialogSettings::getBgColor
+ * @brief DialogSettings::getCanvasBgColor
  * Opens a QColorDialog for the user to select a new bg color
  */
-void DialogSettings::getBgColor(){
+void DialogSettings::getCanvasBgColor(){
 
     m_bgColor = QColorDialog::getColor(
                 m_bgColor, this, tr("Select a background color") );
@@ -344,7 +463,7 @@ void DialogSettings::getBgColor(){
         ui->bgImageSelectEdit->setText("");
         m_appSettings["initBackgroundColor"] = m_bgColor.name();
         m_appSettings["initBackgroundImage"] = "";
-        emit setBgColor(m_bgColor);
+        emit setCanvasBgColor(m_bgColor);
     }
     else {
         // user pressed Cancel
@@ -355,9 +474,9 @@ void DialogSettings::getBgColor(){
 
 
 /**
- * @brief DialogSettings::getBgImage
+ * @brief DialogSettings::getCanvasBgImage
  */
-void DialogSettings::getBgImage(){
+void DialogSettings::getCanvasBgImage(){
     QString m_bgImage = QFileDialog::getOpenFileName(
                 this, tr("Select a background image "), (m_appSettings)["lastUsedDirPath"],
                 tr("All (*);;PNG (*.png);;JPG (*.jpg)")
@@ -365,7 +484,7 @@ void DialogSettings::getBgImage(){
     if (!m_bgImage.isEmpty() ) {
         (m_appSettings)["initBackgroundImage"] = m_bgImage ;
         ui->bgImageSelectEdit->setText((m_appSettings)["initBackgroundImage"]);
-        emit setBgImage();
+        emit setCanvasBgImage();
     }
     else { //user pressed Cancel
 
@@ -374,6 +493,31 @@ void DialogSettings::getBgImage(){
 }
 
 
+
+
+/**
+ * @brief Gets Canvas Update Mode
+ */
+void DialogSettings::getCanvasUpdateMode(const QString &mode){
+    if (!mode.isEmpty() ) {
+        m_appSettings["canvasUpdateMode"] = mode;
+        emit setCanvasUpdateMode(mode);
+    }
+}
+
+
+
+/**
+ * @brief Gets canvas Index Method
+ */
+void DialogSettings::getCanvasIndexMethod(const QString &method){
+    if (!method.isEmpty() ) {
+        m_appSettings["canvasIndexMethod"] = method;
+        emit setCanvasIndexMethod(method);
+    }
+}
+
+
 /**
  * @brief DialogSettings::getNodeColor
  * * Opens a QColorDialog for the user to select a new node color
@@ -578,7 +722,7 @@ void DialogSettings::getEdgeColor(){
     if ( m_edgeColor.isValid()) {
         m_pixmap.fill(m_edgeColor);
         ui->edgeColorBtn->setIcon(QIcon(m_pixmap));
-        (m_appSettings)["initEdgeColor"] = m_edgeColor.name();
+        m_appSettings["initEdgeColor"] = m_edgeColor.name();
         emit setEdgeColor(m_edgeColor, RAND_MAX);
     }
     else {
@@ -589,7 +733,7 @@ void DialogSettings::getEdgeColor(){
 
 /**
  * @brief DialogSettings::getEdgeColorNegative
- * * Opens a QColorDialog for the user to select a new edge color
+ * * Opens a QColorDialog for the user to select a new negative edge color
  */
 void DialogSettings::getEdgeColorNegative(){
     m_edgeColorNegative = QColorDialog::getColor(
@@ -597,8 +741,28 @@ void DialogSettings::getEdgeColorNegative(){
     if ( m_edgeColorNegative.isValid()) {
         m_pixmap.fill(m_edgeColorNegative);
         ui->edgeColorNegativeBtn->setIcon(QIcon(m_pixmap));
-        (m_appSettings)["initEdgeColorNegative"] = m_edgeColorNegative.name();
-        emit setEdgeColor(m_edgeColorNegative, 0);
+        m_appSettings["initEdgeColorNegative"] = m_edgeColorNegative.name();
+        emit setEdgeColor(m_edgeColorNegative, -1);
+    }
+    else {
+        // user pressed Cancel
+    }
+}
+
+
+
+/**
+ * @brief DialogSettings::getEdgeColorZero
+ * * Opens a QColorDialog for the user to select a new zero edge color
+ */
+void DialogSettings::getEdgeColorZero(){
+    m_edgeColorZero = QColorDialog::getColor(
+                m_edgeColorZero, this, tr("Select color for negative Edges") );
+    if ( m_edgeColorZero.isValid()) {
+        m_pixmap.fill(m_edgeColorZero);
+        ui->edgeColorZeroBtn->setIcon(QIcon(m_pixmap));
+        m_appSettings["initEdgeColorZero"] = m_edgeColorZero.name();
+        emit setEdgeColor(m_edgeColorZero, 0);
     }
     else {
         // user pressed Cancel
@@ -625,6 +789,19 @@ void DialogSettings::getEdgeShape(){
 
 
 
+
+/**
+ * @brief Changes the edge offset from source and target nodes
+ * @param size
+ */
+void DialogSettings::getEdgeOffsetFromNode( int offset) {
+    qDebug()<< "DialogSettings::getEdgeOffsetFromNode() - new offset:" << offset;
+    m_appSettings["initEdgeOffsetFromNode"]= QString::number(offset);
+    emit setEdgeOffsetFromNode(offset);
+}
+
+
+
 /**
  * @brief DialogSettings::getEdgeWeightNumbersVisibility
  * @param toggle
