@@ -1,11 +1,11 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer
- version: 2.2
+ version: 2.4
  Written in Qt
  
                          graph.h  -  description
                              -------------------
-    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    copyright         : (C) 2005-2018 by Dimitris B. Kalamaras
     project site      : http://socnetv.org
 
  ***************************************************************************/
@@ -39,16 +39,17 @@
 #include <stack>
 #include <map>
 
-#include "vertex.h"
+#include "graphvertex.h"
 #include "matrix.h"
 #include "parser.h"
 #include "webcrawler.h"
 
 using namespace std;
 
-static const int EDGE_DIRECTED                 = 0;
-static const int EDGE_DIRECTED_OPPOSITE_EXISTS = 1;
-static const int EDGE_RECIPROCAL_UNDIRECTED    = 2;
+static const int EDGE_DIRECTED = 0;
+static const int EDGE_RECIPROCATED = 1;
+static const int EDGE_UNDIRECTED = 2;
+
 
 static const int FILE_GRAPHML           = 1;  // .GRAPHML .XML
 static const int FILE_PAJEK             = 2;  // .PAJ .NET
@@ -101,12 +102,13 @@ class QPointF;
 
 
 
-typedef QList<Vertex*> Vertices;
+typedef QList<GraphVertex*> VList;
 typedef QHash <QString, int> H_StrToInt;
 typedef QHash <long int, long int> H_Int;
+typedef QHash <float, long int> H_f_i;
 typedef QPair <float, bool> pair_f_b;
-typedef QPair <int, pair_f_b > rel_w_bool;
-typedef QHash < int, rel_w_bool > H_edges;
+typedef QPair <int, pair_f_b > pair_i_fb;
+typedef QHash < int, pair_i_fb > H_edges;
 typedef QHash<QString, bool> H_StrToBool;
 typedef QList<int> L_int;
 typedef QVector<int> V_int;
@@ -117,11 +119,13 @@ typedef QVector<QString> V_str;
 struct ClickedEdge {
     int v1;
     int v2;
+    int type;
 };
 
 
 
-typedef pair<int, int> SelectedEdge;
+typedef QPair<int, int> SelectedEdge;
+
 
 
 class GraphDistance
@@ -183,9 +187,9 @@ class GraphDistancesCompare {
 
 /**
  * @brief The Graph class
- * This is the main class for a Graph, used in conjuction with Vertex, Parser and Matrix objects.
+ * This is the main class for a Graph, used in conjuction with GraphVertex, Parser and Matrix objects.
  *   Graph class methods are the interface to various analysis algorithms
- *   Vertex class holds each vertex data (colors, strings, statistics, etc)
+ *   GraphVertex class holds each vertex data (colors, strings, statistics, etc)
  *   Matrix class holds the adjacency matrix of the network.
  *   Parser class loads files of networks.
  */
@@ -220,6 +224,10 @@ public slots:
     void vertexRemoveDummyNode(int);
     void graphLoadedTerminateParserThreads (QString reason);
 
+    void graphSelectionChanged(const QList<int> selectedVertices,
+                               const QList<SelectedEdge> selectedEdges);
+
+
     /** Slots to signals from GraphicsWidget and Parser*/
     void edgeCreate  (const int &v1, const int &v2, const float &weight,
                       const QString &color ,
@@ -239,9 +247,10 @@ public slots:
                                           const bool &signalMW=false) ;
     /** Slots to signals from MainWindow */
 
-    void relationSet(int index=RAND_MAX, const bool notifyMW=true);
+    void relationSet(int relNum=RAND_MAX, const bool notifyMW=true);
     void relationNext();
     void relationPrev();
+
     void canvasSizeSet(const int w, const int h);
     double canvasMaxRadius() const;
     float canvasMinDimension() const;
@@ -249,19 +258,33 @@ public slots:
     double canvasVisibleY(const double &y) const ;
     double canvasRandomX()  const;
     double canvasRandomY()  const;
-    void vertexIsolateFilter ( bool );		//Called by MW to filter orphan vertices
+
+    void vertexIsolatedAllToggle ( const bool &toggle);
     void vertexClickedSet(const int &v);
-    void edgeClickedSet(const int &v1, const int &v2) ;
-    void edgeFilterByWeight (float, bool);		//Called by MW to filter edges over/under a weight
+    void edgeClickedSet(const int &v1, const int &v2, const bool &openMenu=false) ;
+
+    void edgeFilterByWeight (float, bool);
     void edgeFilterByRelation(int relation, bool status);
     void edgeFilterUnilateral(const bool &toggle);
-    void webCrawl(QString, int, int, bool extLinks, bool intLinks);	//Called by MW to start a web crawler...
+
+    void webCrawl(const QString &urlSeed,
+                  const QStringList &urlPatternsIncluded,
+                  const QStringList &urlPatternsExcluded,
+                  const QStringList &linkClasses,
+                  const int &maxNodes,
+                  const int &maxLinksPerPage,
+                  const bool &extLinks,
+                  const bool &intLinks,
+                  const bool &selfLinks,
+                  const bool &delayedRequests);
 
     QString htmlEscaped (QString str) const;
 
 signals:
     /** Signals to MainWindow */
-    void updateProgressDialog(int );
+    void signalProgressBoxCreate(const int max=0, const QString msg="Please wait");
+    void signalProgressBoxKill(const int max=0);
+    void signalProgressBoxUpdate(const int &count=0 );
     void signalGraphModified(const int &graphStatus,
                       const bool &undirected,
                       const int &vertices,
@@ -279,18 +302,18 @@ signals:
 
     void statusMessage (const QString &message);
     void signalDatasetDescription(QString);
-    void signalNodeSizesByOutDegree(bool);
-    void signalNodeSizesByInDegree(bool);
+
     void signalNodeClickedInfo(const int &number=0,
                                     const QPointF &p=QPointF(),
                                     const QString &label=QString::null,
                                     const int &inDegree=0,
                                     const int &outDegree=0,
                                     const float &clc=0);
-    void signalEdgeClickedInfo (const long int &v1=0,
-                                const long int &v2=0,
-                                const float &weight=0,
-                                const bool &undirected=false);
+    void signalEdgeClicked (const int &v1=0,
+                            const int &v2=0,
+                            const float &weight=0,
+                            const int &type=0,
+                            const bool &openMenu=false);
     void signalRelationAddToMW(const QString &newRelation, const bool &changeRelation=true);
     void signalRelationsClear();
     void signalRelationRenamedToMW(const QString newRelName);
@@ -300,26 +323,34 @@ signals:
                                 const int &selectedEdges);
 
     /** Signals to GraphicsWidget */
-    void drawNode( const int &num, const int &size, const QString &nodeShape,
+    void signalDrawNode( const int &num, const int &size, const QString &nodeShape,
                    const QString &nodeColor,
-                   const bool &showNumbers,const bool &numbersInside,
                    const QString &numberColor, const int &numSize,
                    const int &numDistance,
-                   const bool &showLabels, const QString &label,
+                   const QString &label,
                    const QString &labelColor, const int &labelSize,
                    const int &labelDistance,
                    const QPointF &p
                     );
 
-    void eraseNode (long int);						//erase node from GW
-    //call GW to draw an edge
-    void drawEdge ( const int &v1, const int &v2, const float &weight,
-                    const QString &label="",
-                    const QString &color="black",
-                    const int &type=0, const bool arrows=true,
-                    const bool &bezier=false,  const bool &weightNumbers=false);
-    void eraseEdge(const long int &, const long int &);					//emited from edgeRemove() to GW to clear the edge item.
-    void setEdgeVisibility (int, int, int, bool);			// emitted from each Vertex
+    //signal to GW to erase a node
+    void signalRemoveNode (long int );
+
+    //signal GW to draw an edge
+    void signalDrawEdge ( const int &v1,
+                          const int &v2,
+                          const float &weight,
+                          const QString &label="",
+                          const QString &color="black",
+                          const int &type=0,
+                          const bool &drawArrows=true,
+                          const bool &bezier=false,
+                          const bool &weightNumbers=false);
+
+    //signal to GW
+    void signalRemoveEdge(const long int &v1, const long int &v2, const bool &removeOpposite);
+
+    void setEdgeVisibility (int, int, int, bool);			// emitted from each GraphVertex
     void setVertexVisibility(long int, bool);		//notifies GW to disable a node
     void setNodePos(const int &, const qreal &, const qreal &);
     void setNodeSize(const long int &v, const int &size);
@@ -333,7 +364,9 @@ signals:
     void setNodeLabelDistance(const long int &, const int &);
 
     void setEdgeWeight (const long int &v1, const long int &v2, const float &weight);
-    void setEdgeUndirected(const long int &v1, const long int &v2, const float &weight);
+    void signalEdgeType(const long int &v1,
+                        const long int &v2,
+                        const int &type);
     void setEdgeColor(const long int &v1,
                          const long int &v2,
                          const QString &color);
@@ -353,7 +386,7 @@ signals:
 public: 	
     /* INIT AND CLEAR*/
     Graph();
-    void clear();
+    void clear(const QString &reason="");
     ~Graph();			//destroy object
 
     void setSocNetV_Version (QString ver) { VERSION = ver; }
@@ -361,8 +394,8 @@ public:
 
     /*FILES (READ AND WRITE)*/
     QString graphName() const;
-    void graphLoad (const QString m_fileName, const QString m_codecName,
-                    const bool m_showLabels,
+    void graphLoad (const QString m_fileName,
+                    const QString m_codecName,
                     const int format,
                     const int two_sm_mode,
                     const QString delimiter=QString::null);
@@ -409,7 +442,7 @@ public:
 
     int vertexExists(const long int &v1 );
     int vertexExists(const QString &label);
-    void vertexRemove (long int );
+    void vertexRemove (const long int &v1);
 
     void vertexSizeInit (const long int);
     void vertexSizeSet(const long int &v, const int &newsize );
@@ -433,8 +466,7 @@ public:
     void vertexNumberDistanceInit (const int &distance);
     void vertexNumberDistanceSet(const long int &v, const int &newDistance );
     void vertexNumberDistanceSetAll (const int &newDistance);
-    void vertexNumbersInsideNodesSet(bool toggle);
-    void vertexNumbersVisibilitySet(bool toggle);
+
 
     void vertexLabelsVisibilitySet(bool toggle);
     void vertexLabelSizeInit(int newSize);
@@ -449,11 +481,13 @@ public:
     void vertexLabelDistanceSet(const long int &v, const int &newDistance );
     void vertexLabelDistanceAllSet (const int &newDistance);
 
+
+
     void vertexPosSet(const int &v, const int &x, const int &y);
     QPointF vertexPos(const int &v1);
     int vertexClicked() const;
 
-    int vertices(const bool dropIsolates=false, const bool countAll=false) ;
+    int vertices(const bool &dropIsolates=false, const bool &countAll=false, const bool &recount=false) ;
 
     int vertexEdgesOutbound (int i) ;
     int vertexEdgesInbound (int i) ;
@@ -486,7 +520,7 @@ public:
     void layoutForceDirected_FR_moveNodes(const qreal &temperature) ;
 
     qreal layoutForceDirected_FR_temperature(const int iteration) const;
-    qreal computeOptimalDistance(const int &Vertices);
+    qreal computeOptimalDistance(const int &V);
     void compute_angles( const QPointF &Delta,
                          const qreal &dist,
                          qreal &angle1,
@@ -497,12 +531,15 @@ public:
     /* EDGES */
     int edgesEnabled();
     ClickedEdge edgeClicked();
-    float edgeExists(const long &v1, const long &v2, const bool &undirected=false);
+    float edgeExists(const long &v1, const long &v2, const bool &checkReciprocal=false);
 
     void edgeRemove (const long int &v1, const long int &v2,
                      const bool &removeOpposite=false);
     bool edgeSymmetric(const long &v1, const long &v2);
-    void edgeUndirectedSet(const long int &v1, const long int &v2, const float &w);
+    void edgeTypeSet(const long int &v1,
+                     const long int &v2,
+                     const float &w,
+                     const int &dirType=EDGE_DIRECTED);
 
     void edgeWeightSet (const long int &v1, const long int &v2,
                         const float &w,
@@ -525,9 +562,6 @@ public:
     bool graphSaved() const;
     bool graphLoaded() const;
 
-    void graphSelectionChanged(const QList<int> &selectedVertices,
-                               const QList<SelectedEdge> &selectedEdges);
-
     QList<int> graphSelectedVertices() const;
     int graphSelectedVerticesCount() const;
     int graphSelectedVerticesMin() const;
@@ -541,6 +575,8 @@ public:
     float graphDensity();
     bool graphWeighted();
 
+    float graphReciprocity();
+
     bool graphSymmetric();
     void graphSymmetrize();
     void graphSymmetrizeStrongTies(const bool &allRelations=false);
@@ -584,6 +620,8 @@ public:
     void writeMatrixAdjacencyTo(QTextStream& os,
                                 const bool &saveEdgeWeights=true);
 
+    void writeReciprocity( const QString fileName, const bool considerWeights=false);
+
     void writeMatrix(const QString &fileName,
                      const int &matrix=MATRIX_ADJACENCY,
                      const bool &considerWeights=true,
@@ -592,8 +630,15 @@ public:
                      const QString &varLocation="Rows",
                      const bool &simpler=false);
 
+    void writeMatrixHTMLTable(QTextStream &outText, Matrix &M,
+                              const bool &markDiag=true,
+                              const bool &plain=false,
+                              const bool &printInfinity=true,
+                              const bool &dropIsolates=false);
+
     void writeMatrixAdjacency(const QString fileName,
                               const bool &markDiag=true);
+
     void writeMatrixAdjacencyPlot(const QString fileName,
                                       const bool &simpler=false);
 
@@ -608,7 +653,7 @@ public:
                                        const bool &inverseWeights,
                                        const bool &dropIsolates);
 
-    void writeMatrixNumberOfGeodesicsPlainText(const QString &fn,
+    void writeMatrixShortestPathsPlainText(const QString &fn,
                                                const bool &considerWeights,
                                                const bool &inverseWeights);
 
@@ -645,8 +690,8 @@ public:
                                                const QString &varLocation="rows",
                                                const bool &diagonal=false);
 
-    void writeEccentricity(const QString, const bool considerWeights,
-                           const bool inverseWeights, const bool dropIsolates);
+    void writeEccentricity( const QString fileName, const bool considerWeights=false,
+                            const bool inverseWeights=false, const bool dropIsolates=false);
 
  //   friend QTextStream& operator <<  (QTextStream& os, Graph& m);
 
@@ -692,7 +737,8 @@ public:
     void writePrestigePageRank(const QString, const bool Isolates=false);
 
 
-    void writeClusteringHierarchical(const QString &fileName,
+    bool writeClusteringHierarchical(const QString &fileName,
+                                     const QString &varLocation,
                                      const QString &matrix = "Adjancency",
                                      const QString &metric = "Manhattan",
                                      const QString &method = "Complete",
@@ -706,9 +752,8 @@ public:
                                                     const int N,
                                                const bool &dendrogram = false);
 
-    void writeCliqueCensus(
-            const QString fileName, const bool considerWeights
-            );
+    bool writeCliqueCensus( const QString &fileName,
+                            const bool considerWeights);
 
     void writeClusteringCoefficient(const QString, const bool);
 
@@ -720,17 +765,33 @@ public:
 
 
     /* DISTANCES, CENTRALITIES & PROMINENCE MEASURES */
-    int graphDistanceGeodesic(const int, const int,
-                 const bool considerWeights, const bool inverseWeights);
-    int graphDiameter(const bool considerWeights, const bool inverseWeights);
-    float graphDistanceGeodesicAverage(const bool considerWeights,
-                               const bool inverseWeights, const bool dropIsolates);
     int graphConnectedness(const bool updateProgress=false) ;
 
-    void graphMatrixDistancesCreate(const bool &computeCentralities=false,
+    bool graphReachable(const int &v1, const int &v2) ;
+    void graphMatrixReachabilityCreate() ;
+
+    int graphDiameter(const bool considerWeights, const bool inverseWeights);
+
+    int graphDistanceGeodesic(const int v1, const int v2,
+                              const bool considerWeights,
+                              const bool inverseWeights);
+
+    float graphDistanceGeodesicAverage(const bool considerWeights,
+                               const bool inverseWeights, const bool dropIsolates);
+
+    void graphDistanceGeodesicCompute(const bool &computeCentralities=false,
                               const bool &considerWeights=false,
                               const bool &inverseWeights=true,
                               const bool &dropIsolates=false);
+
+    void graphMatrixDistanceGeodesicCreate(const bool &considerWeights=false,
+                                     const bool &inverseWeights=true,
+                                     const bool &dropIsolates=false);
+
+    void graphMatrixShortestPathsCreate(const bool &considerWeights=false,
+                                     const bool &inverseWeights=true,
+                                     const bool &dropIsolates=false) ;
+
     void centralityDegree(const bool &weights=true,
                           const bool &dropIsolates=false);
     void centralityInformation(const bool considerWeights=false,
@@ -750,14 +811,14 @@ public:
 
     /* REACHABILTY AND WALKS */
     int walksBetween(int v1, int v2,int length);
-    void graphWalksMatrixCreate(const int &vertices=0, const int &length=0,
+    void graphWalksMatrixCreate(const int &N=0, const int &length=0,
                                    const bool &updateProgress=false);
     void writeWalksTotalMatrixPlainText(const QString &fn);
     void writeWalksOfLengthMatrixPlainText(const QString &fn, const int &length);
     void writeMatrixWalks (const QString &fn,
                              const int &length=0,
                              const bool &simpler=false);
-    int reachable(const int &v1, const int &v2) ;
+
     QList<int> vertexinfluenceRange(int v1);
     QList<int> vertexinfluenceDomain(int v2);
 
@@ -772,7 +833,8 @@ public:
     int graphCliquesContaining(const int &actor, const int &size=0);
     int graphCliquesOfSize(const int &size );
 
-    void graphClusteringHierarchical(Matrix &STR_EQUIV,
+    bool graphClusteringHierarchical(Matrix &STR_EQUIV,
+                                     const QString &varLocation,
                                      const int &metric,
                                      const int &method,
                                      const bool &diagonal=false,
@@ -784,7 +846,7 @@ public:
     float clusteringCoefficient (const bool updateProgress=false);
 
     bool graphTriadCensus();
-    void triadType_examine_MAN_label(int, int, int, Vertex*,  Vertex*, Vertex* );
+    void triadType_examine_MAN_label(int, int, int, GraphVertex*,  GraphVertex*, GraphVertex* );
     //	void eccentr_JordanCenter(); 				// TODO
 
 
@@ -793,28 +855,33 @@ public:
 
     void layoutRandom();
 
-    void layoutCircularRandom(double x0, double y0, double maxRadius);
+    void layoutRadialRandom(const bool &guides=true);
 
-    void layoutCircularByProminenceIndex(double x0, double y0, double maxRadius,
-                                         int type, const bool considerWeights,
-                                         const bool inverseWeights,
-                                         const bool dropIsolates);
+    void layoutCircular(const double &x0,
+                        const double &y0,
+                        const double &newRadius,
+                        const bool &guides=false);
 
-    void layoutLevelByProminenceIndex(double maxWidth, double maxHeight, int type,
-                                      const bool considerWeights,
-                                      const bool inverseWeights,
-                                      const bool dropIsolates);
 
-    void layoutVerticesSizeByProminenceIndex(int index,
-                                             const bool considerWeights,
-                                             const bool inverseWeights,
-                                             const bool dropIsolates);
+    void layoutByProminenceIndex ( int prominenceIndex, int layoutType,
+                                   const bool considerWeights=false,
+                                   const bool inverseWeights=false,
+                                   const bool dropIsolates=false);
+
+
+    void layoutVertexSizeByIndegree();
+    void layoutVertexSizeByOutdegree();
+
 
     void layoutForceDirectedSpringEmbedder(const int maxIterations);
 
     void layoutForceDirectedFruchtermanReingold(const int maxIterations);
 
-    void layoutForceDirectedKamadaKawai(const int maxIterations);
+    void layoutForceDirectedKamadaKawai(const int maxIterations=500,
+                                        const bool considerWeights=false,
+                                        const bool inverseWeights=false,
+                                        const bool dropIsolates=false,
+                                        const QString &initialPositions="current");
 
     /* CRAWLER */
     void webCrawlTerminateThreads (QString reason);
@@ -822,43 +889,43 @@ public:
     /**RANDOM NETWORKS*/
     void randomizeThings();
 
-    void randomNetErdosCreate (  const int &vert,
+    void randomNetErdosCreate (  const int &N,
                                  const QString &model,
-                                 const int &edges,
-                                 const float &eprob,
+                                 const int &m,
+                                 const float &p,
                                  const QString &mode,
                                  const bool &diag);
 
-    void randomNetRingLatticeCreate (const int &vert, const int &degree,
+    void randomNetRingLatticeCreate (const int &N, const int &degree,
                                       const bool updateProgress=false);
 
-    void randomNetRegularCreate (const int &vert,
+    void randomNetRegularCreate (const int &N,
                                     const int &degree,
                                     const QString &mode,
                                     const bool &diag);
 
-    void randomNetScaleFreeCreate (const int &n,
+    void randomNetScaleFreeCreate (const int &N,
                                     const int &power,
                                     const int &m0,
                                     const int &m,
                                     const float &alpha,
                                     const QString &mode);
 
-    void randomNetSmallWorldCreate(const int &vert, const int &degree,
+    void randomNetSmallWorldCreate(const int &N, const int &degree,
                                    const double &beta, const QString &mode);
 
     int factorial (int);
 
 
-    /**  index stores the real position of each vertex inside m_graph.
+    /**  vpos stores the real position of each vertex inside m_graph.
      *  It starts at zero (0).
      *   We need to know the place of a vertex inside m_graph after adding
      *   or removing many vertices
      */
-    H_Int index;
+    H_Int vpos;
 
     // Stores the number of vertices at distance n from a given vertex
-    H_Int sizeOfNthOrderNeighborhood;
+    H_f_i sizeOfNthOrderNeighborhood;
 
     /* maps have O(logN) lookup complexity */
     /* Consider using tr1::hashmap which has O(1) lookup, but this is not ISO C++ yet :(   */
@@ -866,7 +933,7 @@ public:
 
 protected: 
     // Called from nodeMovement when a timerEvent occurs
-    void timerEvent(QTimerEvent *event);
+    //void timerEvent(QTimerEvent *event);
 
 
 private:
@@ -875,7 +942,7 @@ private:
      * List of pointers to the vertices.
      * A vertex stores all the info: links, colours, etc
     */
-    Vertices m_graph;
+    VList m_graph;
 
     Parser *file_parser;	//file loader threaded class.
 
@@ -895,14 +962,15 @@ private:
                   const QString &color
                   );
 
-    /** methods used by graphMatrixDistancesCreate()  */
-    void BFS(const int &s, const bool &computeCentralities=false,
+    /** methods used by graphDistanceGeodesicCompute()  */
+    void BFS(const int &s, const int &si, const bool &computeCentralities=false,
              const bool &dropIsolates=false);
-    void dijkstra(const int &s, const bool &computeCentralities=false,
+    void dijkstra(const int &s, const int &si, const bool &computeCentralities=false,
                   const bool &inverseWeights=false,
                   const bool &dropIsolates=false);
+
     void minmax(
-                float C, Vertex *v, float &max, float &min,
+                float C, GraphVertex *v, float &max, float &min,
                 int &maxNode, int &minNode
               ) ;
     void resolveClasses (float C, H_StrToInt &discreteClasses, int &classes);
@@ -913,16 +981,26 @@ private:
 
 
     QList<QString> m_relationsList;
-    QList<int>  triadTypeFreqs; 	//stores triad type frequencies
-    QList<int>  m_isolatedVerticesList,m_verticesList, m_graphFileFormatExportSupported;
+
+    QList<int> m_graphFileFormatExportSupported;
+
+    QList<int> triadTypeFreqs; 	//stores triad type frequencies
+
+    QList<int> m_verticesList;
+    QList<int> m_verticesIsolatedList;
+    QList<int> m_verticesInfiniteEccentricity;
+    QList<int> m_verticesSelected;
+
     QSet<int> m_verticesSet;
+
     QList<SelectedEdge> m_selectedEdges;
-    QList<int> m_selectedVertices;
+
     QHash <int, int> influenceRanges, influenceDomains;
     QHash <int, int> m_vertexPairsNotConnected;
     QHash <int, int> m_vertexPairsUnilaterallyConnected;
 
     QMap <int, L_int > m_cliques;
+    QHash <int, QSet<int> > neighboursHash;
 
     QList <float> m_clusteringLevel;
     QMap <int, V_int> m_clustersPerSequence;
@@ -931,12 +1009,12 @@ private:
     QMap<QString, V_int> m_clustersByName;
     QMap<int, V_str> m_clusterPairNamesPerSeq;
 
-    Matrix  TM, DM, sumM, invAM, AM, invM;
+    Matrix  SIGMA, DM, sumM, invAM, AM, invM, WM;
     Matrix XM, XSM, XRM, CLQM;
 
     stack<int> Stack;
 
-    /** used in resolveClasses and graphMatrixDistancesCreate() */
+    /** used in resolveClasses and graphDistanceGeodesicCompute() */
     H_StrToInt discreteDPs, discreteSDCs, discreteCCs, discreteBCs, discreteSCs;
     H_StrToInt discreteIRCCs, discreteECs, discreteEccentricities;
     H_StrToInt discretePCs, discreteICs,  discretePRPs, discretePPs, discreteEVCs;
@@ -956,7 +1034,7 @@ private:
     float meanSDP, varianceSDP;
     float meanPP, variancePP;
     float meanPRP, variancePRP;
-    float minEccentricity, maxEccentricity, sumEccentricity;
+    float minEccentricity, maxEccentricity;
     float minSDP, maxSDP, sumDP, sumSDP, groupDP;
     float minSDC, maxSDC, sumDC, sumSDC, groupDC;
     float minSCC, maxSCC, nomSCC, denomSCC, sumCC, sumSCC, groupCC, maxIndexCC;
@@ -985,7 +1063,7 @@ private:
     int classesPRP, maxNodePRP, minNodePRP;
     int classesPP, maxNodePP, minNodePP;
     int classesEVC, maxNodeEVC, minNodeEVC;
-    int sizeOfComponent;
+    float sizeOfComponent;
 
     /** General & initialisation variables */
 
@@ -993,13 +1071,22 @@ private:
     long int m_totalVertices, m_totalEdges, m_graphDiameter, initVertexSize;
     int initVertexLabelSize, initVertexNumberSize;
     int initVertexNumberDistance, initVertexLabelDistance;
-    bool order, initVertexLabelsVisibility,initVertexNumbersVisibility;
-    bool initVertexNumberInside, initEdgeWeightNumbers, initEdgeLabels;
+    bool order;
+    bool initEdgeWeightNumbers, initEdgeLabels;
     float m_graphAverageDistance, m_graphGeodesicsCount;
     float m_graphDensity;
+    float m_graphReciprocityArc, m_graphReciprocityDyad;
+    int m_graphReciprocityTiesReciprocated;
+    int m_graphReciprocityTiesNonSymmetric;
+    int m_graphReciprocityTiesTotal;
+    int m_graphReciprocityPairsReciprocated;
+    int m_graphReciprocityPairsTotal;
+
+
     int m_graphConnectedness;
     int outboundEdgesVert, inboundEdgesVert, reciprocalEdgesVert;
-    int timerId,  canvasWidth, canvasHeight;
+    //int timerId;
+    int canvasWidth, canvasHeight;
     bool calculatedEdges;
     bool calculatedVertices, calculatedVerticesList, calculatedVerticesSet;
     bool calculatedAdjacencyMatrix, calculatedDistances, calculatedCentralities;
@@ -1008,8 +1095,12 @@ private:
     bool calculatedDP, calculatedDC, calculatedPP;
     bool calculatedIRCC, calculatedIC, calculatedPRP;
     bool calculatedTriad;
-    bool calculatedGraphSymmetry, calculatedGraphDensity, calculatedGraphWeighted;
-    bool m_undirected, m_symmetric, m_isWeighted;
+    bool calculatedGraphSymmetry, calculatedGraphReciprocity;
+    bool calculatedGraphDensity, calculatedGraphWeighted;
+    bool calculatedGraphConnectedness;
+    bool m_undirected, m_symmetric, m_isWeighted, m_graphDisconnected;
+
+    int cliqueCensusRecursion;
 
     QString VERSION, fileName, m_graphName, initEdgeColor, initVertexColor,
         initVertexNumberColor, initVertexLabelColor, initVertexShape;
