@@ -1,11 +1,11 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer
- version: 2.2
+ version: 2.4
  Written in Qt
  
                          graphicswidget.h  -  description
                              -------------------
-    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    copyright         : (C) 2005-2018 by Dimitris B. Kalamaras
     project site      : http://socnetv.org
 
  ***************************************************************************/
@@ -31,37 +31,47 @@
 
 
 #include <QGraphicsView>
-
+#include <QMetaType>
 
 class MainWindow;
-class Node;
-class Edge;
-class NodeNumber;
-class NodeLabel;
-class Guide;
-class EdgeWeight;
-class EdgeLabel;
 
-typedef QHash<QString, Edge*> H_StrToEdge;
-typedef QHash <long int, Node*> H_NumToNode;
+class GraphicsNode;
+class GraphicsEdge;
+class GraphicsNodeNumber;
+class GraphicsNodeLabel;
+class GraphicsGuide;
+class GraphicsEdgeWeight;
+class GraphicsEdgeLabel;
+
+typedef QHash<QString, GraphicsEdge*> H_StrToEdge;
+typedef QHash <long int, GraphicsNode*> H_NumToNode;
 
 using namespace std;
 
 
-typedef pair<int, int> SelectedEdge;
+typedef QPair<int, int> SelectedEdge;
+
+Q_DECLARE_METATYPE(SelectedEdge)
+
+
+
 
 class GraphicsWidget : public QGraphicsView {
     Q_OBJECT
 public:
-    GraphicsWidget(QGraphicsScene*, MainWindow* parent);
+
+    GraphicsWidget(QGraphicsScene *m_scene, MainWindow *m_parent);
+
     ~GraphicsWidget();
     void clear();
 
+    QString createEdgeName(const int &v1, const int &v2, const int &relation=-1);
+
     void setInitNodeSize(int);
+
     void setInitZoomIndex (int);
 
-
-    Node* hasNode(QString text);
+    GraphicsNode* hasNode(QString text);
     bool setMarkedNode(QString text);
 
     QList<QGraphicsItem *> selectedItems();
@@ -71,14 +81,14 @@ public:
     void selectAll();
     void selectNone();
 
-    void removeItem(Edge*);
-    void removeItem(EdgeWeight *edgeWeight);
-    void removeItem(EdgeLabel *edgeLabel);
-    void removeItem(Node*);
-    void removeItem(NodeNumber*);
-    void removeItem(NodeLabel*);
+    void removeItem(GraphicsEdge*);
+    void removeItem(GraphicsEdgeWeight *edgeWeight);
+    void removeItem(GraphicsEdgeLabel *edgeLabel);
+    void removeItem(GraphicsNode*);
+    void removeItem(GraphicsNodeNumber*);
+    void removeItem(GraphicsNodeLabel*);
 
-    void setNumbersInsideNodes(bool);
+    void setNumbersInsideNodes(const bool &toggle);
 
     void setAllItemsVisibility(int, bool);
 
@@ -99,17 +109,16 @@ public slots:
 
     void drawNode(const int &num, const int &nodeSize,
                    const QString &nodeShape, const QString &nodeColor,
-                   const bool &showNumbers, const bool &numberInsideNode,
                    const QString &numberColor, const int &numberSize,
-                  const int &numberDistance,
-                   const bool &showLabels, const QString &nodeLabel,
+                   const int &numberDistance,
+                   const QString &nodeLabel,
                    const QString &labelColor, const int &labelSize,
-                  const int &labelDistance,
+                   const int &labelDistance,
                    const QPointF &p
                     );
-    void eraseNode(const long int &number);
+    void removeNode(const long int &number);
     void setNodeVisibility(long int, bool );	//Called from Graph via MW
-    void nodeClicked(Node *);
+    void nodeClicked(GraphicsNode *);
     void moveNode(const int &num, const qreal &x, const qreal &y);	//Called from Graph when creating random nets.
 
     bool setNodeSize(const long int &nodeNumber, const int &size=0);
@@ -128,7 +137,8 @@ public slots:
     bool setNodeLabel(long int , QString );
     bool setNodeLabelDistance(const long int &, const int &distance=0);
 
-    void drawEdge(const int &source, const int &target,
+    void drawEdge(const int &source,
+                  const int &target,
                   const float &weight,
                   const QString &label="",
                   const QString &color="black",
@@ -136,17 +146,35 @@ public slots:
                   const bool &drawArrows=true,
                   const bool &bezier=false,
                   const bool &weightNumbers=false);
-    void eraseEdge(const long int &source, const long int &target);
+
+    void removeEdge(const long int &source,
+                    const long int &target,
+                    const bool &removeOpposite=false);
+
     void setEdgeVisibility (int relation, int, int, bool);
-    bool setEdgeUndirected(const long int &, const long int &, const float &);
+
+    bool setEdgeDirectionType(const long int &,
+                              const long int &,
+                              const int &dirType=false);
+
     bool setEdgeWeight(const long int &, const long int &, const float &);
+
     void setEdgeLabel(const long int &, const long int&, const QString &);
+
     void setEdgeColor(const long int &, const long int&, const QString &);
-    void edgeClicked(Edge *);
+
+    void edgeClicked(GraphicsEdge *, const bool &openMenu=false);
+
+    void setEdgeOffsetFromNode(const long int &source,
+                               const long int &target,
+                               const int &offset);
+    void setEdgeArrowsVisibility(const bool &toggle);
     void setEdgeWeightNumbersVisibility (const bool &toggle);
     void setEdgeLabelsVisibility(const bool &toggle);
 
-    void startEdge(Node *node);
+    void setEdgeHighlighting(const bool &toggle);
+
+    void startEdge(GraphicsNode *node);
 
     void clearGuides();
     void addGuideCircle( const double&x0, const double&y0, const double&radius);
@@ -165,14 +193,13 @@ signals:
     void userMiddleClicked(const int &, const int &);
     void userClickOnEmptySpace(const QPointF &p);
     void openNodeMenu();
-    void openEdgeMenu();
     void openContextMenu(const QPointF p);
     void userNodeMoved(const int &, const int &, const int &);
     //void userSelectedItems(const int nodes, const int edges);
-    void userSelectedItems(const QList<int> &selectedNodes,
-                           const QList<SelectedEdge> &selectedEdges);
+    void userSelectedItems(const QList<int> selectedNodes,
+                           const QList<SelectedEdge> selectedEdges);
     void userClickedNode(const int &nodeNumber);
-    void userClickedEdge(const int &source, const int &target);
+    void userClickedEdge(const int &source, const int &target, const bool &openMenu=false);
     void zoomChanged(const int);
     void rotationChanged(const int);
     void resized(const int, const int);
@@ -181,6 +208,7 @@ signals:
 
 
 private:
+
     H_NumToNode nodeHash;	//This is used in drawEdge() method
     H_StrToEdge edgesHash; // helper hash to easily find edges
     QList<int> m_selectedNodes;
@@ -188,14 +216,19 @@ private:
     int m_curRelation, m_nodeSize;
     int m_currentRotationAngle;
     int m_zoomIndex, markedNodeOrigSize,markedEdgeSourceOrigSize, markedEdgeTargetOrigSize;
+    int m_edgeMinOffsetFromNode;
     double m_currentScaleFactor;
     qreal fX,fY, factor;
     QString m_nodeLabel, m_numberColor, m_labelColor;
     QString edgeName;
     bool transformationActive;
     bool secondDoubleClick, markedNodeExist, clickedEdgeExists;
-    Node *firstNode, *secondNode, *markedNode1, *markedEdgeSource;
-    Node *markedEdgeTarget, *tempNode ;
+    bool m_nodeNumbersInside, m_nodeNumberVisibility, m_nodeLabelVisibility;
+    bool m_edgeHighlighting;
+    GraphicsNode *firstNode, *secondNode, *markedNode1;
+    GraphicsNode *markedEdgeSource;
+    GraphicsNode *markedEdgeTarget;
+    GraphicsEdge *clickedEdge;
 };
 
 #endif
