@@ -1,11 +1,11 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer
- version: 2.2
+ version: 2.4
  Written in Qt
  
                          mainwindow.h  -  description
                              -------------------
-    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    copyright         : (C) 2005-2018 by Dimitris B. Kalamaras
     blog              : http://dimitris.apeiro.gr
     project site      : http://socnetv.org
 
@@ -24,7 +24,7 @@
 *                                                                              *
 *     You should have received a copy of the GNU General Public License        *
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
-********************************************************************************/
+*******************************************************************************/
 
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
@@ -37,21 +37,21 @@
 */
 
 #include <QMainWindow>
-#include <QGraphicsScene>
 #include <QPrinter>
 #include <QMessageBox>
+#include <QStack>
+#include <QThread>
+#include <QMetaType>
+
+
 #include <math.h>
 
-/** SocNetV specific includes*/
-
 #include "graph.h"
-#include "graphicswidget.h"
 #include "dialogfilteredgesbyweight.h"
-#include "webcrawlerdialog.h"
-#include "dialognodeedit.h"
+
 #include "dialogdatasetselect.h"
 
-static const QString VERSION="2.2";
+static const QString VERSION="2.4";
 
 static const int USER_MSG_INFO=0;
 static const int USER_MSG_CRITICAL=1;
@@ -60,14 +60,13 @@ static const int USER_MSG_CRITICAL_NO_EDGES=3;
 static const int USER_MSG_QUESTION=4;
 static const int USER_MSG_QUESTION_CUSTOM=5;
 
+
 QT_BEGIN_NAMESPACE
+class QGraphicsScene;
 class QMenu;
 class QAction;
 class QCheckBox;
 class QProgressDialog;
-class Edge;
-class Node;
-
 class QPushButton;
 class QToolButton;
 class QLCDNumber;
@@ -80,7 +79,12 @@ QT_END_NAMESPACE
 
 using namespace std;
 
+class GraphicsWidget;
+class GraphicsEdge;
+class GraphicsNode;
 
+class DialogWebCrawler;
+class DialogNodeEdit;
 class DialogPreviewFile;
 class DialogRandErdosRenyi;
 class DialogRandSmallWorld;
@@ -94,6 +98,7 @@ class DialogSettings;
 class TextEditor;
 
 
+
 /**
   \brief The base window of SocNetV contains all widgets and functionality.
 
@@ -106,23 +111,30 @@ class MainWindow : public QMainWindow
 {
     Q_OBJECT
 
+    QThread graphThread;
+
 public:
 
 
     MainWindow(const QString &f);
     ~MainWindow();
 
+    void initGraph();
+    void terminateThreads(const QString &reason);
+    void initView();
     void initActions();
     void initMenuBar();
     void initToolBar();
-    void initToolBox();
-    void initView();
+    void initPanels();
     void initWindowLayout();
     void initSignalSlots();
     QMap<QString, QString> initSettings();
     void saveSettings();
+
     void initApp();
 
+    void initComboBoxes();
+
     void setLastPath(QString filePath);
     QString getLastPath();
     void createFortuneCookies();
@@ -130,9 +142,6 @@ public:
     int activeEdges();
     int activeNodes();
 
-    void createProgressBar(const int &max=0, const QString &msg="Please wait...");
-    void destroyProgressBar(int max=0);
-
 
 
 public slots:
@@ -202,24 +211,34 @@ public slots:
     void slotNetworkRandomScaleFreeDialog();
 
     void slotNetworkRandomScaleFree(const int &newNodes,
-                                 const int &power,
-                                 const int &initialNodes,
-                                 const int &edgesPerStep,
-                                 const float &zeroAppeal,
-                                 const QString &mode);
+                                    const int &power,
+                                    const int &initialNodes,
+                                    const int &edgesPerStep,
+                                    const float &zeroAppeal,
+                                    const QString &mode);
 
     void slotNetworkRandomSmallWorldDialog();
 
     void slotNetworkRandomSmallWorld  (const int &newNodes,
-                                   const int &degree,
-                                   const float &beta,
-                                   const QString &mode,
-                                   const bool &diag);
+                                       const int &degree,
+                                       const float &beta,
+                                       const QString &mode,
+                                       const bool &diag);
 
     void slotNetworkRandomRingLattice();
 
     void slotNetworkWebCrawlerDialog();
-    void slotNetworkWebCrawler(QString, int, int, bool, bool);
+
+    void slotNetworkWebCrawler(const QString &urlSeed,
+                               const QStringList &urlPatternsIncluded,
+                               const QStringList &urlPatternsExcluded,
+                               const QStringList &linkClasses,
+                               const int &maxNodes,
+                               const int &maxLinksPerPage,
+                               const bool &extLinks,
+                               const bool &intLinks,
+                               const bool &selfLinks,
+                               const bool &delayedRequests);
 
     //EDIT MENU
     void slotEditRelationsClear();
@@ -243,7 +262,7 @@ public slots:
                                    const float &clc=0);
     void slotEditNodePosition(const int &nodeNumber, const int &x, const int &y);
     void slotEditNodeAdd();
-    void slotEditNodeAddWithMouse(const QPointF &);
+
     void slotEditNodeFind();
     void slotEditNodeRemove();
     void slotEditNodeOpenContextMenu();
@@ -264,12 +283,13 @@ public slots:
     void slotEditNodeLabelsColor(QColor color=QColor());
     void slotEditNodeLabelDistance(int v1=0, int newSize=0);
 
-    void slotEditEdgeInfoStatusBar (const int &v1,
+    void slotEditEdgeClicked (const int &v1,
                                     const int &v2,
                                     const float &weight,
-                                    const bool &undirected);
+                                    const int &type,
+                                    const bool &openMenu=false);
 
-    void slotEditEdgeOpenContextMenu() ;
+    void slotEditEdgeOpenContextMenu(const QString &str="") ;
     void slotEditEdgeAdd();
     void slotEditEdgeCreate (const int &source, const int &target,
                              const float &weight=1);
@@ -277,7 +297,7 @@ public slots:
     void slotEditEdgeLabel();
     void slotEditEdgeColor();
     void slotEditEdgeWeight();
-    void slotEditEdgeColorAll(QColor color=QColor(), const int &threshold=RAND_MAX);
+    void slotEditEdgeColorAll(QColor color=QColor(), const int threshold=RAND_MAX);
 
     void slotEditEdgeMode(const int &mode);
     void slotEditEdgeSymmetrizeAll();
@@ -295,58 +315,54 @@ public slots:
 
     // LAYOUT MENU
     void slotLayoutRandom();
-    void slotLayoutCircularRandom();
-    void slotLayoutCircularByProminenceIndex();
-    void slotLayoutCircularByProminenceIndex(QString);
-    void slotLayoutNodeSizesByProminenceIndex(QString);
+    void slotLayoutRadialRandom();
+
+    void slotLayoutRadialByProminenceIndex();
+    void slotLayoutRadialByProminenceIndex(QString);
+
     void slotLayoutLevelByProminenceIndex();
     void slotLayoutLevelByProminenceIndex(QString);
-    void slotLayoutGuides(const bool &toggle);
+
+    void slotLayoutNodeSizeByProminenceIndex();
+    void slotLayoutNodeSizeByProminenceIndex(QString);
+
+    void slotLayoutNodeColorByProminenceIndex();
+    void slotLayoutNodeColorByProminenceIndex(QString);
 
     void slotLayoutSpringEmbedder();
     void slotLayoutFruchterman();
     void slotLayoutKamadaKawai();
 
-    void slotLayoutNodeSizesByOutDegree(bool);
-    void slotLayoutNodeSizesByInDegree(bool);
-
     void slotLayoutColorationStrongStructural();
     void slotLayoutColorationRegular();
 
+    void slotLayoutGuides(const bool &toggle);
+
     //ANALYSIS MENU
-    void askAboutWeights();
-    void slotAnalyzeMatrixDistances();
-    void slotAnalyzeMatrixGeodesics();
-    void slotAnalyzeDistance();
-    void slotAnalyzeDistanceAverage();
-    void slotAnalyzeDiameter();
-    void slotAnalyzeEccentricity();
-    void slotAnalyzeStrEquivalenceDissimilaritiesDialog();
-    void slotAnalyzeDissimilaritiesTieProfile(const QString &metric,
-                                               const QString &varLocation,
-                                               const bool &diagonal);
-
-    void slotAnalyzeWalksLength();
-    void slotAnalyzeWalksTotal();
-    void slotAnalyzeReachabilityMatrix();
-    void slotAnalyzeConnectedness();
-
-    void slotAnalyzeStrEquivalenceClusteringHierarchicalDialog();
-    void slotAnalyzeClusteringHierarchical(const QString &matrix,
-                                           const QString &metric,
-                                           const QString &method,
-                                           const bool &diagonal=false,
-                                           const bool &diagram=false);
-    void slotAnalyzeCommunitiesCliqueCensus();
-    void slotAnalyzeClusteringCoefficient();
-    void slotAnalyzeCommunitiesTriadCensus();
+    void askAboutWeights(const bool userTriggered=false);
 
+    void slotAnalyzeReciprocity();
     void slotAnalyzeSymmetryCheck();
     void slotAnalyzeMatrixAdjacencyInverse();
     void slotAnalyzeMatrixAdjacencyTranspose();
     void slotAnalyzeMatrixAdjacencyCocitation();
     void slotAnalyzeMatrixDegree();
     void slotAnalyzeMatrixLaplacian();
+    void slotAnalyzeClusteringCoefficient();
+
+    void slotAnalyzeMatrixDistances();
+    void slotAnalyzeMatrixGeodesics();
+    void slotAnalyzeDistance();
+    void slotAnalyzeDistanceAverage();
+    void slotAnalyzeDiameter();
+    void slotAnalyzeEccentricity();
+
+    void slotAnalyzeWalksLength();
+    void slotAnalyzeWalksTotal();
+    void slotAnalyzeReachabilityMatrix();
+    void slotAnalyzeConnectedness();
+
+
 
     void slotAnalyzeCentralityDegree();
     void slotAnalyzeCentralityCloseness();
@@ -362,23 +378,44 @@ public slots:
     void slotAnalyzePrestigePageRank();
     void slotAnalyzePrestigeProximity();
 
+
+    void slotAnalyzeCommunitiesCliqueCensus();
+    void slotAnalyzeCommunitiesTriadCensus();
+
+
+    void slotAnalyzeStrEquivalenceClusteringHierarchicalDialog();
+    void slotAnalyzeStrEquivalenceClusteringHierarchical(const QString &matrix,
+                                                         const QString &varLocation,
+                                                         const QString &metric,
+                                                         const QString &method,
+                                                         const bool &diagonal=false,
+                                                         const bool &diagram=false);
+    void slotAnalyzeStrEquivalenceDissimilaritiesDialog();
+    void slotAnalyzeStrEquivalenceDissimilaritiesTieProfile(const QString &metric,
+                                               const QString &varLocation,
+                                               const bool &diagonal);
+
     void slotAnalyzeStrEquivalenceSimilarityMeasureDialog();
-    void slotAnalyzeSimilarityMatching(const QString &matrix,
+    void slotAnalyzeStrEquivalenceSimilarityByMeasure(const QString &matrix,
                                const QString &varLocation,
                                const QString &measure,
                                const bool &diagonal);
 
     void slotAnalyzeStrEquivalencePearsonDialog();
-    void slotAnalyzeSimilarityPearson(const QString &matrix,
+    void slotAnalyzeStrEquivalencePearson(const QString &matrix,
                                const QString &varLocation,
                                const bool &diagonal=false);
 
+
     //OPTIONS MENU
     void slotOpenSettingsDialog();
     void slotOptionsNodeNumbersVisibility(bool toggle);
     void slotOptionsNodeNumbersInside(bool toggle);
     void slotOptionsNodeLabelsVisibility(bool toggle);
     void slotOptionsEdgesVisibility(bool toggle);
+    void slotOptionsEdgeOffsetFromNode(const int &offset,
+                                       const int &v1=0,
+                                       const int &v2=0);
     void slotOptionsEdgeLabelsVisibility(bool toggle);
     void slotOptionsEdgeWeightNumbersVisibility(bool toggle);
     void slotOptionsEdgeWeightsDuringComputation(bool);
@@ -386,23 +423,35 @@ public slots:
     void slotOptionsEdgesBezier(bool toggle);
     void slotOptionsEdgeArrowsVisibility(bool toggle);
 
-    void slotOptionsBackgroundColor(QColor color=QColor());
-    void slotOptionsBackgroundImageSelect(bool toggle);
-    void slotOptionsBackgroundImage();
 
-    void slotOptionsAntialiasing(bool );
     void slotOptionsEmbedLogoExporting(bool toggle);
-    void slotOptionsProgressBarVisibility(bool toggle);
+    void slotOptionsProgressDialogVisibility(bool toggle);
     void slotOptionsToolbarVisibility(bool toggle);
     void slotOptionsStatusBarVisibility(bool toggle);
     void slotOptionsLeftPanelVisibility(bool toggle);
     void slotOptionsRightPanelVisibility(bool toggle);
     void slotOptionsDebugMessages(bool toggle);
 
+    void slotOptionsBackgroundColor(QColor color=QColor());
+    void slotOptionsBackgroundImageSelect(bool toggle);
+    void slotOptionsBackgroundImage();
+
+    void slotOptionsCanvasAntialiasing(bool toggle);
+    void slotOptionsCanvasAntialiasingAutoAdjust(const bool &toggle=false);
+    void slotOptionsCanvasSmoothPixmapTransform(const bool &toggle=false);
+    void slotOptionsCanvasSavePainterState(const bool &toggle=false);
+    void slotOptionsCanvasCacheBackground(const bool &toggle=false);
+    void slotOptionsCanvasEdgeHighlighting(const bool &toggle=false);
+
+    void slotOptionsCanvasUpdateMode(const QString &mode);
+    void slotOptionsCanvasIndexMethod(const QString &method);
+
+
     //HELP MENU
     void slotHelpTips();
     void slotHelp();
-    void slotHelpCheckUpdates();
+    void slotHelpCheckUpdateDialog();
+    void slotHelpCheckUpdateParse(QNetworkReply *reply);
     void slotHelpCreateTips();
     void slotHelpAbout();
     void slotAboutQt();
@@ -432,10 +481,13 @@ public slots:
     void toolBoxAnalysisStrEquivalenceSelectChanged(int);
     void toolBoxAnalysisProminenceSelectChanged(int);
     void toolBoxAnalysisCommunitiesSelectChanged(int);
-    void toolBoxLayoutByIndexButtonPressed();
-    void toolBoxLayoutForceDirectedButtonPressed();
+    void toolBoxLayoutByIndexApplyBtnPressed();
+    void toolBoxLayoutForceDirectedApplyBtnPressed();
 
 
+    void slotProgressBoxCreate(const int &max=0, const QString &msg="Please wait...");
+    void slotProgressBoxDestroy(const int &max=0);
+
 protected:
     void resizeEvent( QResizeEvent * );
     void closeEvent( QCloseEvent* ce );
@@ -446,14 +498,16 @@ signals:
     void signalRelationAddAndChange(const QString &relName, const bool &changeRelation=true);
 
 private:
+
     QGraphicsScene *scene;
     GraphicsWidget *graphicsWidget;
-    Graph activeGraph;
+
+    Graph *activeGraph;
 
     QMap<QString,QString> appSettings;
 
     DialogFilterEdgesByWeight m_DialogEdgeFilterByWeight;
-    WebCrawlerDialog m_WebCrawlerDialog;
+    DialogWebCrawler *m_WebCrawlerDialog;
     DialogDataSetSelect m_datasetSelectDialog;
 
     DialogNodeEdit *m_nodeEditDialog;
@@ -473,7 +527,6 @@ private:
     QString userSelectedCodecName;
 
 
-
     QList<TextEditor *> m_textEditors;
 
     QPrinter *printer;
@@ -483,7 +536,7 @@ private:
 
     QComboBox *editRelationChangeCombo;
 
-    QProgressDialog *progressDialog;
+    QStack<QProgressDialog *> progressDialogs;
 
     QMenu *importSubMenu, *exportSubMenu, *editMenu, *analysisMenu, *helpMenu;
     QMenu *optionsMenu, *colorOptionsMenu, *edgeOptionsMenu, *nodeOptionsMenu;
@@ -491,9 +544,9 @@ private:
     QMenu *cohesionMenu, *strEquivalenceMenu, *communitiesMenu, *connectivityMenu;
     QMenu *matrixMenu;
     QMenu *networkMenu, *randomNetworkMenu, *filterMenu, *recentFilesSubMenu;
-    QMenu *randomLayoutMenu, *circleLayoutMenu, *levelLayoutMenu, *physicalLayoutMenu;
+    QMenu *randomLayoutMenu, *layoutRadialProminenceMenu, *layoutLevelProminenceMenu;
+    QMenu *layoutForceDirectedMenu, *layoutNodeSizeProminenceMenu, *layoutNodeColorProminenceMenu;
     QMenu *colorationMenu;
-    QCheckBox  *toolBoxNodeSizesByOutDegreeBx,*toolBoxNodeSizesByInDegreeBx, *toolBoxLayoutGuidesBx;
     QComboBox *toolBoxEditNodeSubgraphSelect, *toolBoxEditEdgeModeSelect,
     *toolBoxEditEdgeSymmetrizeSelect, *toolBoxAnalysisCohesionSelect,
     *toolBoxAnalysisStrEquivalenceSelect,
@@ -502,8 +555,7 @@ private:
     QComboBox *toolBoxLayoutByIndexSelect, *toolBoxLayoutByIndexTypeSelect;
     QComboBox *toolBoxLayoutForceDirectedSelect;
 
-    QPushButton *editNodeAddBt, *editEdgeAddBt, *removeNodeBt, *editEdgeRemoveBt;
-    QPushButton *toolBoxLayoutByIndexButton, *toolBoxLayoutForceDirectedButton;
+    QPushButton *toolBoxLayoutByIndexApplyButton, *toolBoxLayoutForceDirectedApplyButton;
 
     QAction *zoomInAct,*zoomOutAct,*editRotateRightAct,*editRotateLeftAct, *editResetSlidersAct ;
     QToolButton *zoomInBtn,*zoomOutBtn,*rotateLeftBtn,*rotateRightBtn, *resetSlidersBtn ;
@@ -542,15 +594,17 @@ private:
             *editNodeNumbersColorAct,*editNodeLabelsColorAct,
             *editEdgesCocitationAct;
     QAction *optionsEdgeThicknessPerWeightAct, *optionsEdgeWeightNumbersAct, *optionsEdgesVisibilityAct;
-    QAction *optionsEdgeArrowsAct, *drawEdgesBezier,*considerEdgeWeightsAct;
+    QAction *optionsEdgeArrowsAct, *drawEdgesBezier,*optionsEdgeWeightConsiderAct;
     QAction *optionsEdgeLabelsAct;
     QAction *backgroundImageAct,*helpAboutApp, *helpAboutQt, *helpApp, *tipsApp;
     QAction *helpCheckUpdatesApp;
     QAction *openSettingsAct;
     QAction *webCrawlerAct;
 
-    QAction *netDensity, *analyzeGraphSymmetryAct, *analyzeGraphDistanceAct, *averGraphDistanceAct,
-            *analyzeMatrixDistancesGeodesicAct, *analyzeMatrixGeodesicsAct, *analyzeGraphDiameterAct, *analyzeGraphEccentricityAct;
+    QAction *netDensity, *analyzeGraphReciprocityAct, *analyzeGraphSymmetryAct;
+    QAction *analyzeGraphDistanceAct, *averGraphDistanceAct;
+    QAction *analyzeMatrixDistancesGeodesicAct, *analyzeMatrixGeodesicsAct;
+    QAction *analyzeGraphDiameterAct, *analyzeGraphEccentricityAct;
     QAction *analyzeStrEquivalenceTieProfileDissimilaritiesAct;
     QAction *analyzeGraphWalksAct,*analyzeGraphWalksTotalAct, *analyzeMatrixReachabilityAct, *analyzeGraphConnectednessAct;
     QAction *analyzeCommunitiesCliquesAct, *clusteringCoefAct, *analyzeCommunitiesTriadCensusAct;
@@ -562,18 +616,33 @@ private:
     QAction *cDegreeAct, *cInDegreeAct, *cClosenessAct, *cInfluenceRangeClosenessAct,
             *cBetweennessAct, *cInformationAct, *cEigenvectorAct, *cPageRankAct,
             *cStressAct, *cPowerAct, *cEccentAct, *cProximityPrestigeAct;
-    QAction *randLayoutAct, *randCircleLayoutAct, *layoutGuidesAct;
-    QAction *layoutCircular_DC_Act, *layoutCircular_DP_Act,
-    *layoutCircular_CC_Act, *layoutCircular_SC_Act, *layoutCircular_EC_Act,
-    *layoutCircular_PC_Act, *layoutCircular_BC_Act, *layoutCircular_IC_Act,
-    *layoutCircular_IRCC_Act,*layoutCircular_PRP_Act, *layoutCircular_PP_Act;
-    QAction *layoutLevel_DC_Act, *layoutLevel_DP_Act,
-    *layoutLevel_CC_Act, *layoutLevel_SC_Act, *layoutLevel_EC_Act,
-    *layoutLevel_PC_Act, *layoutLevel_BC_Act, *layoutLevel_IC_Act,
-    *layoutLevel_IRCC_Act,*layoutLevel_PRP_Act, *layoutLevel_PP_Act;
+    QAction *layoutRandomAct, *layoutRandomRadialAct, *layoutGuidesAct;
+    QAction *layoutRadialProminence_DC_Act, *layoutRadialProminence_DP_Act,
+    *layoutRadialProminence_CC_Act, *layoutRadialProminence_SC_Act, *layoutRadialProminence_EC_Act,
+    *layoutRadialProminence_PC_Act, *layoutRadialProminence_BC_Act, *layoutRadialProminence_IC_Act,
+    *layoutRadialProminence_EVC_Act,
+    *layoutRadialProminence_IRCC_Act,*layoutRadialProminence_PRP_Act, *layoutRadialProminence_PP_Act;
+    QAction *layoutLevelProminence_DC_Act, *layoutLevelProminence_DP_Act,
+    *layoutLevelProminence_CC_Act, *layoutLevelProminence_SC_Act, *layoutLevelProminence_EC_Act,
+    *layoutLevelProminence_PC_Act, *layoutLevelProminence_BC_Act, *layoutLevelProminence_IC_Act,
+    *layoutLevelProminence_EVC_Act,
+    *layoutLevelProminence_IRCC_Act,*layoutLevelProminence_PRP_Act, *layoutLevelProminence_PP_Act;
+    QAction *layoutNodeSizeProminence_DC_Act, *layoutNodeSizeProminence_DP_Act,
+    *layoutNodeSizeProminence_CC_Act, *layoutNodeSizeProminence_SC_Act, *layoutNodeSizeProminence_EC_Act,
+    *layoutNodeSizeProminence_PC_Act, *layoutNodeSizeProminence_BC_Act, *layoutNodeSizeProminence_IC_Act,
+    *layoutNodeSizeProminence_EVC_Act,
+    *layoutNodeSizeProminence_IRCC_Act,*layoutNodeSizeProminence_PRP_Act, *layoutNodeSizeProminence_PP_Act;
+    QAction *layoutNodeColorProminence_DC_Act, *layoutNodeColorProminence_DP_Act,
+    *layoutNodeColorProminence_CC_Act, *layoutNodeColorProminence_SC_Act, *layoutNodeColorProminence_EC_Act,
+    *layoutNodeColorProminence_PC_Act, *layoutNodeColorProminence_BC_Act, *layoutNodeColorProminence_IC_Act,
+    *layoutNodeColorProminence_EVC_Act,
+    *layoutNodeColorProminence_IRCC_Act,*layoutNodeColorProminence_PRP_Act, *layoutNodeColorProminence_PP_Act;
+
+
     QAction *strongColorationAct, *regularColorationAct;
-    QAction *springLayoutAct, *FRLayoutAct;
-    QAction *nodeSizesByOutDegreeAct,  *nodeSizesByInDegreeAct;
+    QAction *layoutFDP_Eades_Act, *layoutFDP_FR_Act;
+    QAction *layoutFDP_KamadaKawai_Act;
+
     QAction *editRelationNextAct, *editRelationPreviousAct, *editRelationAddAct;
     QAction *editRelationRenameAct;
     enum { MaxRecentFiles = 5 };
@@ -589,22 +658,43 @@ private:
     //QString VERSION;
 
     bool markedNodesExist;
-    bool considerWeights, inverseWeights, askedAboutWeights;
+    bool inverseWeights, askedAboutWeights;
     float randomErdosEdgeProb;
     QString initFileCodec;
 
-    QLabel *rightPanelEdgesLabel, *rightPanelSelectedEdgesLabel, *rightPanelNetworkTypeLabel ;
-    QLabel *rightPanelClickedEdgeHeaderLabel;
-    QLCDNumber *rightPanelClickedNodeInDegreeLCD, *rightPanelClickedNodeOutDegreeLCD;
-    QLCDNumber *rightPanelClickedNodeLCD, *rightPanelClickedNodeClucofLCD;
-    QLCDNumber *rightPanelNodesLCD, *rightPanelEdgesLCD, *rightPanelDensityLCD;
-    QLCDNumber *rightPanelSelectedNodesLCD, *rightPanelSelectedEdgesLCD;
-    QLCDNumber *rightPanelClickedEdgeSourceLCD, *rightPanelClickedEdgeTargetLCD;
-    QLCDNumber *rightPanelClickedEdgeWeightLCD;
+    QLabel *rightPanelNetworkTypeLCD ;
+    QLabel *rightPanelEdgesLabel;
+
+    QLabel *rightPanelClickedNodeHeaderLabel;
+    QLabel *rightPanelNodesLCD;
+    QLabel *rightPanelEdgesLCD;
+    QLabel *rightPanelDensityLCD;
+
+    QLabel *rightPanelClickedNodeLCD;
+    QLabel *rightPanelClickedNodeInDegreeLCD;
+    QLabel *rightPanelClickedNodeOutDegreeLCD;
+    QLabel *rightPanelClickedNodeClucofLCD;
+
+    QLabel *rightPanelSelectedNodesLCD;
+    QLabel *rightPanelSelectedEdgesLCD;
+    QLabel *rightPanelSelectedEdgesLabel;
+
+
+    QLabel *rightPanelClickedEdgeNameLabel;
+    QLabel *rightPanelClickedEdgeNameLCD;
+    QLabel *rightPanelClickedEdgeWeightLabel;
+    QLabel *rightPanelClickedEdgeWeightLCD;
+    QLabel *rightPanelClickedEdgeReciprocalWeightLabel;
+    QLabel *rightPanelClickedEdgeReciprocalWeightLCD;
+
     QDateTime actualDateTime, actualDate, actualTime;
     QTime eTime;     //used  to time algorithms.
 
 
+    QNetworkAccessManager *http;
+    QNetworkRequest request;
+    QNetworkReply *reply;
+
 };
 #endif 
 
