@@ -1,11 +1,11 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer 
- version: 2.2
+ version: 2.4
  Written in Qt
 
                         matrix  -  description
                              -------------------
-    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    copyright         : (C) 2005-2018 by Dimitris B. Kalamaras
     project site      : http://socnetv.org
 
  ***************************************************************************/
@@ -1174,7 +1174,7 @@ bool Matrix::ludcmp (Matrix &a, const int &n, int indx[], float &d) {
  * @param a: input matrix a as the LU decomposition of A, returned by the routine ludcmp
  * @param n: input size of matrix
  * @param indx: input vector, records the row permutation, returned by the routine ludcmp
- * @param b: input array as the right-hand side vector B, and ouput with the solution vector X
+ * @param b: input array as the right-hand side vector B, and output with the solution vector X
  * @return:
  *
  * a, n, and indx are not modified by this routine and can be left in place for
@@ -1257,6 +1257,55 @@ Matrix& Matrix::inverse(Matrix &a)
 
 
 
+
+
+/**
+ * @brief Computes and returns the solution of the set of n linear equations A·x = b
+ * Allows A.solve(b)
+ *
+ * @param b vector
+ * @return
+ */
+bool Matrix::solve(float b[])
+{
+
+    Matrix *A = new Matrix(this->rows(), this->cols());
+
+    *A = *this;
+
+
+    int n=rows();
+    float d;
+
+    int indx[n];
+
+    qDebug () << "Matrix::solve() - solving A x  - size " << n;
+    if (n==0) {
+        return false;
+    }
+    if ( ! ludcmp(*A,n,indx,d) )
+    { //  Decompose the matrix just once.
+        qDebug () << "Matrix::solve() - matrix a singular - RETURN";
+        return false ;
+    }
+
+    qDebug () << "Matrix::solve() - call lubksb";
+    lubksb(*A, n, indx, b);
+    qDebug () << "Matrix::solve() - finished!";
+
+    return true;
+}
+
+
+/**
+ * @brief Computes the dissimilarities matrix of the variables (rows, columns, both)
+ * of this matrix using the user defined metric
+ * @param metric
+ * @param varLocation
+ * @param diagonal
+ * @param considerWeights
+ * @return
+ */
 Matrix& Matrix::distancesMatrix(const int &metric,
                         const QString varLocation,
                         const bool &diagonal,
@@ -1267,7 +1316,8 @@ Matrix& Matrix::distancesMatrix(const int &metric,
 
     qDebug()<< "Matrix::distancesMatrix() -"
             <<"metric"<< metric
-            << "varLocation"<< varLocation;
+            << "varLocation"<< varLocation
+            << "diagonal"<<diagonal;
 
     int N = 0;
     float sum = 0;
@@ -1281,9 +1331,8 @@ Matrix& Matrix::distancesMatrix(const int &metric,
 
         QVector<float> mean (N,0); // holds mean values
 
-        qDebug()<< "Matrix::distancesMatrix() -"
-                <<"input matrix";
-        //printMatrixConsole(true);
+        qDebug()<< "Matrix::distancesMatrix() - input matrix:";
+        //this->printMatrixConsole();
 
         for (int i = 0 ; i < N ; i++ ) {
             sum = 0 ;
@@ -1293,8 +1342,11 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                 max = 0;
                 for (int j = 0 ; j < N ; j++ ) {
 
-                    if (!diagonal && (i==j || k==j))
+//                    qDebug() <<  "(i,j)" << i<< ","<<j << "(k,j)" << k<<","<<j;
+
+                    if (!diagonal && (i==j || k==j)) {
                         continue;
+                    }
 
                     switch (metric) {
                     case METRIC_JACCARD_INDEX:
@@ -1316,7 +1368,8 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                             distTemp = RAND_MAX;
                         }
                         else {
-                            distTemp += ( item(i,j) - item(k,j) )*( item(i,j) - item(k,j) ); //compute (x * y)^2
+//                            qDebug() <<  item(i,j) << "-" << item(k,j) <<"^2";
+                            distTemp += ( item(i,j) - item(k,j) )*( item(i,j) - item(k,j) ); //compute (x - y)^2
                         }
                         break;
                     case METRIC_MANHATTAN_DISTANCE:
@@ -1324,7 +1377,7 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                             distTemp = RAND_MAX;
                         }
                         else {
-                            distTemp += fabs( item(i,j) - item(k,j) ); //compute |x * y|
+                            distTemp += fabs( item(i,j) - item(k,j) ); //compute |x - y|
                         }
                         break;
                     case METRIC_CHEBYSHEV_MAXIMUM:
@@ -1368,9 +1421,9 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                 }
 
 
-                qDebug() << "distTemp("<<i+1<<","<<k+1<<") =" << distTemp
+//                qDebug() << "distTemp("<<i+1<<","<<k+1<<") =" << distTemp
+//                         << "matchRatio("<<i+1<<","<<k+1<<") =" << distance;
 
-                         << "matchRatio("<<i+1<<","<<k+1<<") =" << distance;
                 T->setItem(i,k, distance);
                 T->setItem(k,i, distance);
 
@@ -1424,7 +1477,7 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                             distTemp = RAND_MAX;
                         }
                         else {
-                            distTemp += ( item(j,i) - item(j,k) )*( item(j,i) - item(j,k) ); //compute (x * y)^2
+                            distTemp += ( item(j,i) - item(j,k) )*( item(j,i) - item(j,k) ); //compute (x - y)^2
                         }
                         break;
                     case METRIC_MANHATTAN_DISTANCE:
@@ -1432,7 +1485,7 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                             distTemp = RAND_MAX;
                         }
                         else {
-                            distTemp += fabs( item(j,i) - item(j,k) ); //compute |x * y|
+                            distTemp += fabs( item(j,i) - item(j,k) ); //compute |x - y|
                         }
                         break;
                     case METRIC_CHEBYSHEV_MAXIMUM:
@@ -1475,9 +1528,10 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                 }
 
 
-                qDebug() << "distTemp("<<i+1<<","<<k+1<<") =" << distTemp
+//                qDebug() << "distTemp("<<i+1<<","<<k+1<<") =" << distTemp
+
+//                         << "distance("<<i+1<<","<<k+1<<") =" << distance;
 
-                         << "distance("<<i+1<<","<<k+1<<") =" << distance;
                 T->setItem(i,k, distance);
                 T->setItem(k,i, distance);
 
@@ -1547,7 +1601,7 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                             distTemp = RAND_MAX;
                         }
                         else {
-                            distTemp += ( CM.item(j,i) - CM.item(j,k) )*( CM.item(j,i) - CM.item(j,k) ); //compute (x * y)^2
+                            distTemp += ( CM.item(j,i) - CM.item(j,k) )*( CM.item(j,i) - CM.item(j,k) ); //compute (x - y)^2
                         }
                         break;
                     case METRIC_MANHATTAN_DISTANCE:
@@ -1555,7 +1609,7 @@ Matrix& Matrix::distancesMatrix(const int &metric,
                             distTemp = RAND_MAX;
                         }
                         else {
-                            distTemp += fabs( CM.item(j,i) - CM.item(j,k) ); //compute |x * y|
+                            distTemp += fabs( CM.item(j,i) - CM.item(j,k) ); //compute |x - y|
                         }
                         break;
                     case METRIC_CHEBYSHEV_MAXIMUM:
@@ -1601,9 +1655,10 @@ Matrix& Matrix::distancesMatrix(const int &metric,
 
 
 
-                qDebug() << "distTemp("<<i+1<<","<<k+1<<") =" << distTemp
+//                qDebug() << "distTemp("<<i+1<<","<<k+1<<") =" << distTemp
+
+//                         << "matchRatio("<<i+1<<","<<k+1<<") =" << distance;
 
-                         << "matchRatio("<<i+1<<","<<k+1<<") =" << distance;
                 T->setItem(i,k, distance);
                 T->setItem(k,i, distance);
 
@@ -1618,7 +1673,8 @@ Matrix& Matrix::distancesMatrix(const int &metric,
     else {
 
     }
-
+    qDebug() << "Matrix::distancesMatrix() - FINISHED - Returning matrix:";
+    //T->printMatrixConsole();
     return *T;
 }
 
@@ -1700,7 +1756,7 @@ Matrix& Matrix::similarityMatrix(Matrix &AM,
                         magn_k  += AM.item(k,j) * AM.item(k,j); //compute |y|^2
                         break;
                     case METRIC_EUCLIDEAN_DISTANCE:
-                        matches += ( AM.item(i,j) - AM.item(k,j) )*( AM.item(i,j) - AM.item(k,j) ); //compute (x * y)^2
+                        matches += ( AM.item(i,j) - AM.item(k,j) )*( AM.item(i,j) - AM.item(k,j) ); //compute (x - y)^2
                         break;
                     default:
                         break;
@@ -1802,7 +1858,7 @@ Matrix& Matrix::similarityMatrix(Matrix &AM,
                         magn_k  += AM.item(j,k) * AM.item(j,k); //compute |y|^2
                         break;
                     case METRIC_EUCLIDEAN_DISTANCE:
-                        matches += ( AM.item(j,i) - AM.item(j,k) )*( AM.item(j,i) - AM.item(j,k) ); //compute (x * y)^2
+                        matches += ( AM.item(j,i) - AM.item(j,k) )*( AM.item(j,i) - AM.item(j,k) ); //compute (x - y)^2
                         break;
                     default:
                         break;
@@ -1917,7 +1973,7 @@ Matrix& Matrix::similarityMatrix(Matrix &AM,
                         magn_k  += CM.item(j,k) * CM.item(j,k); //compute |y|^2
                         break;
                     case METRIC_EUCLIDEAN_DISTANCE:
-                        matches += ( CM.item(j,i) - CM.item(j,k) )*( CM.item(j,i) - CM.item(j,k) ); //compute (x * y)^2
+                        matches += ( CM.item(j,i) - CM.item(j,k) )*( CM.item(j,i) - CM.item(j,k) ); //compute (x - y)^2
                         break;
                     default:
                         break;
@@ -2384,7 +2440,9 @@ QTextStream& operator <<  (QTextStream& os, Matrix& m){
 
 
 /**
- * @brief  Prints this matrix a HTML table
+ * @brief  Prints this matrix as HTML table
+ * This has the problem that the real actorNumber != elementLabel i.e. when we
+ * have deleted a node/vertex
  * @param os
  * @param debug
  * @return
@@ -2394,7 +2452,7 @@ bool Matrix::printHTMLTable(QTextStream& os,
                             const bool &plain,
                             const bool &printInfinity){
     qDebug() << "Matrix::printHTMLTable()";
-    int actorNumber=0, rowCount = 0;
+    int elementLabel=0, rowCount = 0;
     float maxVal, minVal, element;
     bool hasRealNumbers=false;
 
@@ -2411,17 +2469,17 @@ bool Matrix::printHTMLTable(QTextStream& os,
         os << "<span class=\"header\">" << qSetFieldWidth(5) << right << "A/A";
         os <<  fixed << qSetFieldWidth(10) << right ;
         for (int r = 0; r < cols(); ++r) {
-            actorNumber = r+1;
-            os << actorNumber;
+            elementLabel = r+1;
+            os << elementLabel;
         }
         os << qSetFieldWidth(0) << "</span>"<< endl;
 
         for (int r = 0; r < rows(); ++r) {
-            actorNumber = r+1;
+            elementLabel = r+1;
             rowCount++;
 
             os << "<span class=\"header\">" << qSetFieldWidth(5) << right;
-            os << actorNumber;
+            os << elementLabel;
             os << qSetFieldWidth(0) << "</span>";
 
             for (int c = 0; c < cols(); ++c) {
@@ -2453,9 +2511,9 @@ bool Matrix::printHTMLTable(QTextStream& os,
 
     // print first/header row
     for (int r = 0; r < cols(); ++r) {
-        actorNumber = r+1;
+        elementLabel = r+1;
         os << "<th>"
-                << actorNumber
+                << elementLabel
                 << "</th>";
 
     }
@@ -2466,12 +2524,12 @@ bool Matrix::printHTMLTable(QTextStream& os,
     // print rows
     rowCount = 0;
     for (int r = 0; r < rows(); ++r) {
-        actorNumber = r+1;
+        elementLabel = r+1;
         rowCount++;
         os << "<tr class=" << ((rowCount%2==0) ? "even" :"odd" )<< ">";
 
         os <<"<td class=\"header\">"
-               << actorNumber
+               << elementLabel
                << "</td>";
 
         for (int c = 0; c < cols(); ++c) {
@@ -2528,12 +2586,12 @@ bool Matrix::printHTMLTable(QTextStream& os,
  * @return
  */
 bool Matrix::printMatrixConsole(bool debug){
-    //qDebug() << "Matrix::printMatrixConsole() debug " << debug ;
+    qDebug() << "Matrix::printMatrixConsole() - debug " << debug
+             << "matrix rows" << rows()<< "cols"<< cols();
     QTextStream out ( (debug ? stderr : stdout) );
 
     for (int r = 0; r < rows(); ++r) {
         for (int c = 0; c < cols(); ++c) {
-
             if ( item(r,c) < RAND_MAX  ) {
                 out <<  qSetFieldWidth(12) << qSetRealNumberPrecision(3)
                      <<  forcepoint << fixed<<right
@@ -2554,3 +2612,25 @@ bool Matrix::printMatrixConsole(bool debug){
 }
 
 
+/**
+ * @brief  Checks if matrix is ill-defined (contains at least an inf element)
+ * @return
+ */
+bool Matrix::illDefined(){
+    qDebug() << "Matrix::illDefined() " ;
+
+    for (int r = 0; r < rows(); ++r) {
+        for (int c = 0; c < cols(); ++c) {
+            if ( item(r,c) < RAND_MAX  ) {
+            }
+            else {
+                qDebug() << "Matrix::illDefined() - matrix ill-defined: TRUE" ;
+                return true;
+
+            }
+        }
+    }
+    return false;
+}
+
+
