@@ -1,11 +1,11 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer
- version: 2.2
+ version: 2.4
  Written in Qt
 
                           matrix.h  -  description
                              -------------------
-    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    copyright         : (C) 2005-2018 by Dimitris B. Kalamaras
     project site      : http://socnetv.org
 
  ***************************************************************************/
@@ -223,6 +223,8 @@ public:
 
     Matrix& inverse(Matrix &a);
 
+    bool solve(float b[]);
+
     bool ludcmp (Matrix &a, const int &n, int indx[], float &d ) ;
 
     void lubksb (Matrix &a, const int &n, int indx[], float b[]);
@@ -252,9 +254,10 @@ public:
                         const bool &printInfinity=true);
     bool printMatrixConsole(bool debug=true);
 
+    bool illDefined();
+
 private:
     MatrixRow *row;
-    int m_Actors;
     int m_rows;
     int m_cols;
 
